#include "unit_tests/suite/lumix_unit_tests.h"

#include "engine/array.h"
#include "engine/hash_map.h"
#include "engine/debug/debug.h"


using namespace Lumix;


namespace
{
	void UT_insert(const char* params)
	{
		DefaultAllocator main_allocator;
		Debug::Allocator allocator(main_allocator);
		HashMap<i32, i32> hash_table(allocator);

		LUMIX_EXPECT(hash_table.empty());

		i32 values[10] = {
			1, 2, 3, 4, 5, 6, 7, 8, 9, 10
		};

		for (i32 val : values)
		{
			hash_table.insert(val, val);
		}

		for (i32 val : values)
		{
			LUMIX_EXPECT(hash_table[val] == val);
		}
	};

	void UT_array(const char* params)
	{
		DefaultAllocator main_allocator;
		Debug::Allocator allocator(main_allocator);
		HashMap<i32, Array<int> > hash_table(allocator);

		LUMIX_EXPECT(hash_table.empty());
	};

	void UT_clear(const char* params)
	{
		DefaultAllocator main_allocator;
		Debug::Allocator allocator(main_allocator);
		HashMap<i32, i32> hash_table(allocator);

		LUMIX_EXPECT(hash_table.empty());

		const i32 COUNT = 20;

		for (i32 i = 0; i < COUNT; i++)
		{
			hash_table.insert(i, i);
		}

		for (i32 i = 0; i < COUNT; i++)
		{
			LUMIX_EXPECT(hash_table[i] == i);
		}

		LUMIX_EXPECT(!hash_table.empty());

		hash_table.clear();

		LUMIX_EXPECT(hash_table.empty());

		hash_table.rehash(8);

		for (i32 i = 0; i < COUNT; i++)
		{
			hash_table.insert(i, i);
		}
	};

	void UT_constIterator(const char* params)
	{
		typedef HashMap<i32, i32> HashTableType;

		DefaultAllocator main_allocator;
		Debug::Allocator allocator(main_allocator);
		HashTableType hash_table(allocator);

		LUMIX_EXPECT(hash_table.empty());

		i32 values[] = {
			1, 2, 3, 4, 5, 6, 7, 8, 9, 10
		};

		for (i32 val : values)
		{
			hash_table.insert(val, val);
		}

		for (i32 val : values)
		{
			LUMIX_EXPECT(hash_table[val] == val);
		}

		const HashTableType& const_hash_table = hash_table;
		for (auto const_it = const_hash_table.begin(); const_it != const_hash_table.end(); ++const_it)
		{
			LUMIX_EXPECT(const_it.value() == values[const_it.key() - 1]);
		}

		hash_table.insert(15, 15);
		hash_table.insert(26, 26);// 15 and 26 collide
		hash_table.rehash(64);
	}
}

REGISTER_TEST("unit_tests/engine/hash_map/insert", UT_insert, "")
REGISTER_TEST("unit_tests/engine/hash_map/array", UT_array, "")
REGISTER_TEST("unit_tests/engine/hash_map/clear", UT_clear, "")
REGISTER_TEST("unit_tests/engine/hash_map/constIterator", UT_constIterator, "")

